import express from 'express';
import bcrypt from 'bcrypt';
import multer from 'multer';
import fs from 'fs';
import path from 'path';
import verifyToken from './verifyToken.js';
import { fileURLToPath } from 'url';
import { v4 as uuidv4 } from 'uuid'; // مكتبة لتوليد معرفات فريدة ${uuidv4()}${ext}
import db from './db.js'


const router = express.Router();


const __filename = fileURLToPath(import.meta.url);
const __dirname = path.dirname(__filename);


// إعداد التخزين للصور
const storage = multer.diskStorage({
  destination: (req, file, cb) => {
    cb(null, 'uploadsadmin/');
  },
  filename: (req, file, cb) => {
        const ext = path.extname(file.originalname);
        cb(null, `${Date.now()}-${uuidv4()}${ext}`);
  },
});
const upload = multer({ storage });

router.get('/', async (req, res) => {
    try {
     // console.log('Fetching all admins...');
      const [rows] = await db.query('SELECT * FROM admins');
      res.json(rows);
    } catch (error) {
      res.status(500).json({ message: 'حدث خطأ أثناء جلب المستخدمين.' });
    }
  });

  router.get('/details', verifyToken, async (req, res) => {
    try {
      const [rows] = await db.execute(
        'SELECT * FROM admins WHERE id = ?',
        [req.user.id]
      );
  
      if (rows.length === 0) {
        return res.status(404).json({ message: 'User not found' });
      }
  
      res.json(rows[0]);
    } catch (error) {
      res.status(500).json({ message: 'Internal server error' });
    }
  });
  

// تعديل بيانات المستخدم مع الصورة وكلمة المرور
router.put('/edit/:id', upload.single('image'), async (req, res) => {
  const { id } = req.params;
  const { username, email, role, phone, password, is_visible_a } = req.body;
  const newImage = req.file ? req.file.filename : null;

  try {
    // جلب الصورة القديمة
    const [rows] = await db.query('SELECT image FROM admins WHERE id = ?', [id]);
    const oldImage = rows[0]?.image;

    // إذا كان هناك صورة جديدة، حذف الصورة القديمة
    if (newImage && oldImage) {
      const oldImagePath = path.join('uploadsadmin', oldImage);
      if (fs.existsSync(oldImagePath)) {
        fs.unlinkSync(oldImagePath);
      }
    }

    // تحديث كلمة المرور إذا تم تقديمها
    let hashedPassword = null;
    if (password) {
      hashedPassword = await bcrypt.hash(password, 10);
    }

    // تحديث بيانات المستخدم
    const query = `
      UPDATE admins 
      SET username = ?, email = ?, role = ?, phone = ?, is_visible_a = ?, ${newImage ? 'image = ?,' : ''} ${hashedPassword ? 'password = ?,' : ''} registration_date = NOW() 
      WHERE id = ?
    `;
    const values = [username, email, role, phone, is_visible_a];
    if (newImage) values.push(newImage);
    if (hashedPassword) values.push(hashedPassword);
    values.push(id);

    await db.query(query, values);
    res.json({ message: 'تم تعديل بيانات المستخدم بنجاح!' });
  } catch (error) {
    res.status(500).json({ message: 'حدث خطأ أثناء تعديل المستخدم.' });
  }
});

// حذف المستخدمين
router.delete('/delete/:id', async (req, res) => {
  const { id } = req.params;

  try {
    // الحصول على صورة المستخدم
    const [rows] = await db.execute('SELECT image FROM admins WHERE id = ?', [id]);
    const userImage = rows[0]?.image;

    // حذف المستخدم من قاعدة البيانات
    const deleteQuery = 'DELETE FROM admins WHERE id = ?';
    await db.execute(deleteQuery, [id]);

    // حذف الصورة إذا كانت موجودة
    if (userImage) {
      const imagePath = path.join(__dirname, '../uploadsadmin', userImage);
      if (fs.existsSync(imagePath)) {
        fs.unlinkSync(imagePath); // حذف الصورة
      }
    }

    res.status(200).json({ message: 'User and associated image deleted successfully' });
  } catch (error) {
    res.status(500).json({ message: 'Error deleting user or image' });
  }
});



export default router;
