import express from 'express';
import multer from 'multer';
import fs from 'fs';
import path from 'path';
import { v4 as uuidv4 } from 'uuid'; // مكتبة لتوليد معرفات فريدة ${uuidv4()}${ext}
import db from './db.js'

const router = express.Router();

// إعداد التخزين للصور
const storage = multer.diskStorage({
  destination: (req, file, cb) => {
    cb(null, 'uploadsads/');
  },
  filename: (req, file, cb) => {
    const ext = path.extname(file.originalname);
    cb(null, `${Date.now()}-${uuidv4()}${ext}`);
  },
});


// التحقق من نوع الملفات
const fileFilter = (req, file, cb) => {
  const allowedExtensions = ['.png', '.jpg', '.jpeg', '.gif', '.webp'];
  const ext = path.extname(file.originalname).toLowerCase();

  if (allowedExtensions.includes(ext)) {
    cb(null, true); // السماح برفع الملف
  } else {
    cb(new Error('الملف غير مدعوم. يُسمح فقط برفع الصور (PNG, JPG, JPEG, GIF, WEBP).'), false);
  }
};

// إعداد Multer مع التحقق من نوع الملفات وتحديد الحد الأقصى لحجم الملفات
const upload = multer({
  storage,
  fileFilter,
  limits: { fileSize: 5 * 1024 * 1024 }, // الحد الأقصى لحجم الملف (5 ميجابايت)
});

// إضافة إعلان جديد
router.post('/add', 
  upload.fields([
    { name: 'img1', maxCount: 1 },
    { name: 'img2', maxCount: 1 },
    { name: 'img3', maxCount: 1 }
  ]), 
    async (req, res) => {
  const {
    adresads,
    textads,
    phoneadsid,
    useradsid,
    dateend,
    alaqsamid,
    alfroid,
    alfrooid,
    modelid,
    cash,
    areasid,
    citiesid,
    is_visible,
  } = req.body;
  const img1 = req.files.img1 ? req.files.img1[0].filename : null;
  const img2 = req.files.img2 ? req.files.img2[0].filename : null;
  const img3 = req.files.img3 ? req.files.img3[0].filename : null;

  try {
    const query = `
      INSERT INTO ads (adresads, textads, phoneadsid, useradsid, img1, img2, img3, dateend, alaqsamid, alfroid, alfrooid, modelid, cash, areasid, citiesid, is_visible, datestart)
      VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, NOW())
    `;
    await db.execute(query, [
      adresads,
      textads,
      phoneadsid,
      useradsid,
      img1,
      img2,
      img3,
      dateend,
      alaqsamid || null,
      alfroid || null,
      alfrooid || null,
      modelid || null,
      cash || null,
      areasid || null,
      citiesid || null,
      is_visible || 1,
    ]);
    res.status(201).json({ message: 'تم إضافة الإعلان بنجاح!' });
  } catch (error) {
    res.status(500).json({ message: 'حدث خطأ أثناء إضافة الإعلان.' });
  }
});

// إدارة الأخطاء عند رفع الملفات
router.use((err, req, res, next) => {
  if (err instanceof multer.MulterError) {
    // خطأ خاص بـ Multer (مثل تجاوز الحجم)
    res.status(400).json({ message: `خطأ في رفع الملف: ${err.message}` });
  } else if (err) {
    // أخطاء أخرى (مثل نوع الملف غير المدعوم)
    res.status(400).json({ message: err.message });
  } else {
    next();
  }
});

// تعديل إعلان
router.put('/edit/:id', async (req, res) => {
  const { id } = req.params;
  const {
    adresads, textads, phoneadsid, useradsid, dateend, alaqsamid, alfroid, alfrooid, modelid, cash, areasid, citiesid, is_visible,
  } = req.body;

  try {
    const query = `
      UPDATE ads
      SET adresads = ?, textads = ?, phoneadsid = ?, useradsid = ?, dateend = ?, alaqsamid = ?, alfroid = ?, alfrooid = ?, modelid = ?, cash = ?, areasid = ?, citiesid = ?, is_visible = ?
      WHERE id = ?
    `;
    await db.query(query, [adresads, textads, phoneadsid, useradsid, dateend, alaqsamid, alfroid, alfrooid, modelid, cash, areasid, citiesid, is_visible, id]);
    res.json({ message: 'تم تعديل الإعلان بنجاح!' });
  } catch (error) {
    res.status(500).json({ message: 'حدث خطأ أثناء تعديل الإعلان.' });
  }
});

// جلب جميع الإعلانات
router.get('/', async (req, res) => {
  try {
    const [rows] = await db.query('SELECT * FROM ads');
    res.json(rows);
  } catch (error) {
    res.status(500).json({ message: 'حدث خطأ أثناء جلب الإعلانات.' });
  }
});

// حذف إعلان
router.delete('/delete/:id', async (req, res) => {
  const { id } = req.params;

  try {
    // جلب الصور المرتبطة بالإعلان من قاعدة البيانات
    const [rows] = await db.query('SELECT img1, img2, img3 FROM ads WHERE id = ?', [id]);
    const ad = rows[0];

    if (!ad) {
      return res.status(404).json({ message: 'الإعلان غير موجود.' });
    }

    // حذف الصور المرتبطة إذا كانت موجودة
    if (ad.img1) {
      const img1Path = path.join('uploadsads', ad.img1);
      if (fs.existsSync(img1Path)) {
        fs.unlinkSync(img1Path);
      }
    }
    if (ad.img2) {
      const img2Path = path.join('uploadsads', ad.img2);
      if (fs.existsSync(img2Path)) {
        fs.unlinkSync(img2Path);
      }
    }
    if (ad.img3) {
      const img3Path = path.join('uploadsads', ad.img3);
      if (fs.existsSync(img3Path)) {
        fs.unlinkSync(img3Path);
      }
    }

    // حذف الإعلان من قاعدة البيانات
    await db.query('DELETE FROM ads WHERE id = ?', [id]);

    res.json({ message: 'تم حذف الإعلان بنجاح!' });
  } catch (error) {
    res.status(500).json({ message: 'حدث خطأ أثناء حذف الإعلان.' });
  }
});


export default router;